/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef pplite_Integer_hh
#define pplite_Integer_hh 1

#include "Integer_fwd.hh"

#include "globals.hh"
#include <vector>

#if PPLITE_USE_FLINT_INTEGERS

#include "FLINT_Integer.hh"

#else // !PPLITE_USE_FLINT_INTEGERS

#include "GMP_Integer.hh"

#endif // !PPLITE_USE_FLINT_INTEGERS

namespace pplite {

inline void
normalize2(const Integer& x, const Integer& y,
           Integer& n_x, Integer& n_y) {
  // FIXME: consider declaring it static PPLITE_TLS
  Integer gcd;
  gcd_assign(gcd, x, y);
  exact_div_assign(n_x, x, gcd);
  exact_div_assign(n_y, y, gcd);
}

inline std::ostream&
operator<<(std::ostream& s, const Integer& i) {
  i.print(s);
  return s;
}

inline std::istream&
operator>>(std::istream& s, Integer& i) {
  if (!i.ascii_load(s))
    s.setstate(std::ios::failbit);
  return s;
}

using Integers = std::vector<Integer>;

} // namespace pplite

#endif // !defined(pplite_Integer_hh)
