/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef pplite_Poly_widen_hh
#define pplite_Poly_widen_hh 1

#include "globals.hh"
#include "Poly.hh"
#include "Poly_templ.hh"
#include <map>
#include <vector>

namespace pplite {

namespace h79_widen {

void h79_widen(Poly_Impl& x, const Poly_Impl& y);

} // namespace h79_widen

namespace bhrz03_widen {

// The convergence certificate.
class Cert {
public:
  Cert() = default;
  Cert(const Cert&) = default;
  Cert& operator=(const Cert&) = default;
  Cert(Cert&&) noexcept = default;
  Cert& operator=(Cert&&) noexcept = default;
  ~Cert() = default;

  explicit Cert(const Poly_Impl& ph);

  int compare(const Cert& y) const;
  int compare(const Poly_Impl& ph) const;

  // Returns true if ph is stabilizing wrt this certificate,
  // i.e., this is strictly bigger than ph in the ordering.
  bool is_stabilizing(const Poly_Impl& ph) const { return compare(ph) == 1; }

  bool check_inv() const;

private:
  // Note: default values are for the universe 0-dim polyhedron.
  dim_type affine_dim = 0;
  dim_type lin_space_dim = 0;
  // Non-redundant skeleton constraints.
  dim_type num_sk_cons = 0;
#ifndef NDEBUG
  // Just for debugging purposes.
  dim_type equalities = 0;
#endif
  /*
    A map providing a partial description of the supports of non-redundant
    strict inequalities; the meaning of a pair <j, k> is that there are
    k non-redundant strict inequalities whose support has cardinality j.
  */
  std::map<dim_type, dim_type> support_cards;
  // Non-redundant skeleton (closure) points.
  dim_type num_sk_points = 1;
  /*
    A vector containing, for each index `0 <= i < space_dim',
    the number of non-redundant rays in a generator system of the
    polyhedron having exactly `i' null coordinates.
  */
  std::vector<dim_type> num_rays_null_coord;
};

NOTHROW_MOVES(Cert);

void bhrz03_widen(Poly_Impl& x, const Poly_Impl& y);

} // namespace bhrz03_widen

} // namespace pplite

#endif // !defined(pplite_Poly_widen_hh)

