/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef pplite_Scalar_Prod_hh
#define pplite_Scalar_Prod_hh 1

#include "globals.hh"
#include "Low_Level_Stats.hh"
#include "Integer.hh"
#include "Con.hh"
#include "Gen.hh"

namespace pplite {

struct Scalar_Prod {

  static void
  assign(Integer& z,
         const Linear_Expr& expr, const Integer& inhomo, const Gen& g) {
    z = inhomo * g.impl().inhomo;
    expr.scalar_prod_add(z, g.linear_expr());
  }
  static void
  assign(Integer& z, const Con& c, const Gen& g) {
    assign(z, c.linear_expr(), c.inhomo_term(), g);
  }
  static void
  assign(Integer& z, const Gen& g, const Con& c) { assign(z, c, g); }

  static int
  sign(const Con& c, const Gen& g) {
    Integer z;
    assign(z, c, g);
    return sgn(z);
  }
  static int
  sign(const Gen& g, const Con& c) { return sign(c, g); }

}; // struct Scalar_Prod

} // namespace pplite

#endif // !defined(pplite_Scalar_Prod_hh)
