/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef pplite_globals_hh
#define pplite_globals_hh 1

#ifndef PPLITE_CONVERSION_TIME_STATS
// Produces number of calls and computation time for conversion:
// output occurs at end of program execution.
#define PPLITE_CONVERSION_TIME_STATS 0
#endif

#ifndef PPLITE_LOW_LEVEL_COUNTERS
// Produces number of calls to low level operations
// (scalar products, linear combinations, saturation row inclusion tests,
// unions and population counts); output occurs at end of program execution.
// Requires setting of PPLITE_CONV_TIME_STATS.
#define PPLITE_LOW_LEVEL_COUNTERS 0
#endif

#ifndef PPLITE_NOISY_CONVERSION
// Produces noisy stats after each iteration of the conversion procedure.
#define PPLITE_NOISY_CONVERSION 0
#endif

#ifndef PPLITE_DIVERGING_NS_WIDEN
// Turns on the blind extension of the sat-based H79 widening,
// which compromises the convergence guarantee for NNC polyhedra.
#define PPLITE_DIVERGING_NS_WIDEN 0
#endif

#ifdef PPLITE_THREAD_SAFE
#define PPLITE_TLS thread_local
#else
#define PPLITE_TLS
#endif

#include <vector>
#include <cstddef>
#include <climits>

#define NOTHROW_DEFAULT_CTOR(Type)                                  \
  static_assert(std::is_nothrow_default_constructible<Type>::value, \
                #Type " should be noexcept default constructible")

#define NOTHROW_MOVES(Type)                                      \
  static_assert(std::is_nothrow_move_constructible<Type>::value, \
                #Type " should be noexcept move constructible"); \
  static_assert(std::is_nothrow_move_assignable<Type>::value,    \
                #Type " should be noexcept move assignable")

#define NOTHROW_DEFAULT_AND_MOVES(Type) \
  NOTHROW_DEFAULT_CTOR(Type);           \
  NOTHROW_MOVES(Type)

#ifndef NDEBUG
#define DEBUG_PRINT(msg) { std::cerr << (msg) << std::endl; }
#else
#define DEBUG_PRINT(msg)
#endif

namespace pplite {

using dim_type = int;

enum class Topol { CLOSED, NNC };
enum class Spec_Elem { EMPTY, UNIVERSE };

constexpr dim_type not_a_dim() { return -1; }

template <typename T>
constexpr dim_type sizeof_to_bits(const T size) {
  return size * CHAR_BIT;
}

using Dims = std::vector<dim_type>;

enum class TV_Bool { DONT_KNOW, FALSE, TRUE };

using Counter = unsigned long long;

} // namespace pplite

#endif // !defined(pplite_globals_hh)
